/*
 * ConversionHandler.h
 *
 *  Created on: 23. 2. 2014
 *      Author: tomas
 */

#ifndef ALGORITHMCHOOSER_H_
#define ALGORITHMCHOOSER_H_

#include <list>

#include <AlibException.h>
#include <AutomatonFactory.h>
#include <automaton/AutomatonParser.h>
#include <GrammarFactory.h>
#include <grammar/GrammarParser.h>
#include <regexp/RegExpParser.h>

#include "../fa2re/StateElimination.h"
#include "../fa2re/BrzozowskiAlgebraic.h"
#include "../re2fa/Glushkov.h"
#include "../re2fa/Thompson.h"
#include "../re2fa/Brzozowski.h"
#include "../fa2rg/fa2lrg/FAtoLRGConverter.h"
#include "../fa2rg/fa2rrg/FAtoRRGConverter.h"
#include "../rg2fa/LRGtoFAConverter.h"
#include "../rg2fa/RRGtoFAConverter.h"
#include "../rg2re/RRGAlgebraic.h"
#include "../rg2re/LRGAlgebraic.h"
#include "../re2rg/re2rrg/BrzozowskiDerivationRRG.h"
#include "../lrg2rrg/LeftToRightRegularGrammar.h"
#include "../rrg2lrg/RightToLeftRegularGrammar.h"
#include "../re2rg/re2rrg/GlushkovRRG.h"

namespace conversions
{

class ConversionHandler
{
public:
    enum TAlgorithm
    {
        DEFAULT,

        /* FA to RE */
        BRZOZOWSKI_ALGEBRAIC,
        STATE_ELIMINATION,

        /* FA to RG */
        FA_LRG,
        FA_RRG,

        /* RE to FA */
        BRZOZOWSKI_DERIVATION,
        THOMPSON,
        GLUSHKOV,

        /* RE to RG */
        BRZOZOWSKI_RRG,
        GLUSHKOV_RRG,

        /* RG to FA */
        RG_FA,

        /* RG to RE */
        // BRZOZOWSKI_ALGEBRAIC,
        // STATE_ELIMINATION,

        /* RG to RG */
        LRG_RRG,
    };

    enum TFormalism
    {
        FINITE_AUTOMATA,
        REGULAR_GRAMMAR,
        REGULAR_EXPRESSION,
        UNKNOWN,
    };

    ConversionHandler( std::list<sax::Token> & tokens, const std::string & target, const std::string & algorithm );
    void convert( std::ostream & out );

private:
    TFormalism m_source, m_target;
    TAlgorithm m_algorithm;
    std::list<sax::Token> & m_tokens;

    TFormalism parseFormalismFromString( const std::string & _target ) const;
    TAlgorithm parseAlgorithmFromString( const std::string & _algorithm ) const;
    TAlgorithm getDefaultAlgorithm( void ) const;
    void convertAutomata( std::ostream & out );
    void convertGrammar( std::ostream & out );
    void convertRegExp( std::ostream & out );
    bool checkAlgorithm( void ) const;


};

} /* namespace conversions */

#endif /* ALGORITHMCHOOSER_H_ */
