#include "FAtoRRGConverter.h"

using namespace std;
using namespace automaton;
using namespace grammar;

namespace conversions
{

FAtoRRGConverter::FAtoRRGConverter( const FSM & automaton ) : AbstractFAtoRRGConverter( automaton )
{

}

FAtoRRGConverter::~FAtoRRGConverter( void )
{

}

RightRegularGrammar FAtoRRGConverter::convert( void )
{
    for( const auto & symbol : m_fsm.getInputAlphabet( ) )
        m_grammar.addTerminalSymbol( symbol );

    StateNonterminalMap symbolMap( m_fsm, m_grammar );

    // step 2 - create set of P in G
    for( const auto & transition : m_fsm.getTransitions( ) )
    {
        // 2a
        list<Symbol> leftSide, rightSide;
        leftSide.push_back( symbolMap.getNonTerminal( transition.getFrom( ) ) );
        rightSide.push_back( transition.getInput( ) );
        rightSide.push_back( symbolMap.getNonTerminal( transition.getTo( ) ) );
        m_grammar.addRule( Rule( leftSide, rightSide ) );

        // 2b
        if( isInSet( transition.getTo( ), m_fsm.getFinalStates( ) ) )
        {
            list<Symbol> leftSide, rightSide;
            leftSide.push_back( symbolMap.getNonTerminal( transition.getFrom( ) ) );
            rightSide.push_back( transition.getInput( ) );
            m_grammar.addRule( Rule( leftSide, rightSide ) );
        }
    }

    // step 3 - set start symbol of G
    m_grammar.setStartSymbol( symbolMap.getNonTerminal( * m_fsm.getInitialStates( ).begin( ) ) );

    // step 4
    if( isInSet( State( m_grammar.getStartSymbol( ).getSymbol( ) ), m_fsm.getFinalStates( ) ) )
    {
        // 4a
        if( ! m_grammar.isNonTerminalOnRightSideOfAnyRule( m_grammar.getStartSymbol( ) ) )
        {
            list<Symbol> leftSide, rightSide;
            leftSide.push_back( m_grammar.getStartSymbol( ) );
            m_grammar.addRule( Rule( leftSide, rightSide ) );
        }
        else // 4b
        {
            Symbol newStart = m_grammar.createUniqueNonTerminalSymbol( m_grammar.getStartSymbol( ).getSymbol( ), false );

            for( const auto & rule : m_grammar.getRules( ) )
            {
                if( *( rule.getLeftSide( ).begin( ) ) == m_grammar.getStartSymbol( ) )
                {
                    list<Symbol> leftSide, rightSide( rule.getRightSide( ) );
                    leftSide.push_back( newStart );
                    m_grammar.addRule( Rule( leftSide, rightSide ) );
                }
            }
            m_grammar.setStartSymbol( newStart );

            list<Symbol> leftSide, rightSide;
            leftSide.push_back( newStart );
            Rule rr( leftSide, rightSide );
            m_grammar.addRule( Rule( leftSide, rightSide ) );
        }
    }

    return m_grammar;
}

} /* namespace conversions */
