/*
 * RegularEquationSolver.h
 *
 *  Created on: 4. 3. 2014
 *      Author: tomas
 */

#ifndef REGULAREQUATIONSOLVER_H_
#define REGULAREQUATIONSOLVER_H_

#include <map>
#include <deque>
#include <queue>

#include <alphabet/Symbol.h>
#include <AlibException.h>
#include <regexp/RegExpElement.h>

#include "../../include/macros.h"

#include "RegExpOptimize.h"

namespace conversions
{

class RegularEquationSolver
{
public:
    RegularEquationSolver( void );
    virtual ~RegularEquationSolver( void );

    void addSymbol( const alphabet::Symbol & symb );
    void addEquation( const alphabet::Symbol & from, const alphabet::Symbol & to, const regexp::RegExpElement* eq );
    void addEquation( const alphabet::Symbol & from, const regexp::RegExpElement* eq );
    regexp::RegExp solve( const alphabet::Symbol & solveFor );

protected:
    virtual regexp::RegExpElement* eliminate( void ) = 0;

    /**
     * Runs BFS to determine depth of symbols in equation system
     * @see m_symbolsByDepth
     * @return symbols sorted by depth starting with solveFor having depth 0
     */
    void symbolsByDepth( const alphabet::Symbol & solveFor );

    /**
     * Stores transitions from nonterminal to nonterminal, eg A = 2A + 2B + 1C
     */
    std::map<alphabet::Symbol, std::map<alphabet::Symbol, regexp::RegExpElement*>> m_eqTransition;

    /**
     * Stores equation not going to particular nonterminal, eg A = 01*
     */
    std::map<alphabet::Symbol, regexp::RegExpElement*> m_eqFinal;
    std::set<alphabet::Symbol> m_symbols;
    std::deque<alphabet::Symbol> m_symbolsByDepth;
};

} /* namespace conversions */

#endif /* REGULAREQUATIONSOLVER_H_ */
